//=========================================
// File name: definitions.js
//-----------------------------------------
// Project : QuizFaber {TEXT_QF_VERSION}
// Licence : {TEXT_QF_SCRIPT_LICENCE}
// Author  : Luca Galli
// Email   : {TEXT_QF_EMAIL}
//-----------------------------------------
// Constants and classes for managing quizzes
//=========================================

const QMAKE_VERS = '{QUIZFABER_VERSION}';
const QMAKE_PROGRAMNAME = "{QUIZFABER_NAME}";
const QMAKE_URL = '{QUIZFABER_WEBSITE}';
const RESULT_PAGE_URL = "results.html";
const FINAL_PAGE_URL = "final.html";
const INITIAL_PAGE_URL = "index.html";

const QMAKE_MULTIANS = 1;  /* risposta multipla */
const QMAKE_MULTIANS_WITH_POINT = 101; /* risposta multipla con punteggio */
const QMAKE_BOOLEAN = 2;  /* risposta booleana */
const QMAKE_OPENANS = 3;  /* risposta aperta */
const QMAKE_FILLGAP = 4;  /* da riempire i vuoti */
const QMAKE_MATCHING = 5;  /* associazione tra coppie di parole */
const QMAKE_CUSTOMQST = 6;  /* tipologia di domanda personalizzata dall'utente */

const QMAKE_NO_VALUATION = 0;                /* nessuna valutazione */
const QMAKE_VALUATION_NEAR_TO_MARK = 1;      /* valutazione vicino al voto */
const QMAKE_VALUATION_REPLACE_MARK = 2;      /* valutazione che sostituisce il voto */
const QMAKE_VALUATION_REPLACE_RESULTS = 3;   /* valutazione che sostituisce tutti i risultati (non solo il voto) */
const QMAKE_VALUATION_WITH_LINK = 4;		 /* valutazione che rimanda ad una pagina */

const QMAKE_COMPUTE_MARK_BASIC = 0;
const QMAKE_COMPUTE_MARK_PRECISE = 1;
const QMAKE_COMPUTE_MARK_CUSTOM = 2;

const QMAKE_REPNOT_PT = 1;
const QMAKE_REPNOT_SIMPLE = 2;

const QMAKE_NO_SAVE = 0;
const QMAKE_SEND_WEBAPP = 4;
const QMAKE_SEND_NODEJS = 5;
const QMAKE_SEND_NODEJS_LOCAL = 6;  

const QUIZ_KEYWORD = "{TEXT_QUIZ_KEYWORD}";

const USER_LOGIN_ANONYMOUS = "anonymous";
const USER_EMAIL_ANONYMOUS = "anonymous@anonymous.nowhere";

const options =
{
	name:"{TEXT_QUIZ_TITLE}",
	title: "{TEXT_QUIZ_DESCRIPTION}",
	id: {QUIZ_ID},
	numOfQuestions: {QUIZ_NUM_OF_QUESTIONS},
	notValuatedQuestionsArray: {QUIZ_NOT_VALUATED_QST_ARRAY},
	maxtime: {QUIZ_MAX_TIME},
	maxmark: {QUIZ_MAX_MARK},
	minmark: {QUIZ_MIN_MARK},
	roundmark: {QUIZ_ROUND_MARK},
	questSlide: {QUIZ_QUEST_SLIDE},
	lockRightAns: {QUIZ_LOCK_RIGHT_ANS},
	computeMarkFnType: {QUIZ_COMPUTE_MARK_FN_TYPE},
	randQuest: {QUIZ_QUEST_RANDOM},
	allowChangeChoiceAlways: {QUIZ_ALLOW_CHANGE_CHOICE_ALWAYS},
	verifyQuestBeforeNext: true,
	verifyAtLeastOneChoice: {QUIZ_VERIFY_AT_LEAST_ONE_CHOICE},
	radioChoiceInsteadOkButton: {QUIZ_RADIO_CHOICE_INSTEAD_OK_BUTTON},
	silentBeforeEndQuiz: {QUIZ_SILENT_ANS},
	uniqueOkButton: {QUIZ_UNIQUE_OK_BUTTON},
	uniqueOkButtonPos: '{QUIZ_UNIQUE_OK_BUTTON_POS}',
	showTimeout: true,
	allowAbandonFromQuiz: {QUIZ_ALLOW_ABANDON},
	valuateAfterAbandon: {QUIZ_VALUATE_AFTER_ABANDON},
	needValuateQuiz: {QUIZ_NEED_VALUATE_QUIZ},
	showFullReport: {QUIZ_REPORT_FULL},
	showFullQstReport: {QUIZ_REPORT_FULL_QST},
	showFullAnsReport: {QUIZ_REPORT_FULL_ANS},
	showFullRemReport: {QUIZ_REPORT_FULL_REM},
	reportNotation: {QUIZ_REPORT_NOTATION},
	reviewQuizAtTheEnd: {QUIZ_REVIEW_QUIZ_AT_THE_END},
	msgForNoValuateQuiz : '{QUIZ_MSG_FOR_NO_VALUATE}',
	matchingLockLeftCol: {QUIZ_MATCHING_LOCK_LEFT_COL},
	markPercentage: {QUIZ_REPORT_MARK_PERCENTAGE},
	showPrintButton: {QUIZ_SHOW_PRINT_BUTTON},
	showLinkButton: {QUIZ_SHOW_LINK_BUTTON},
	clearAlwaysHistory: true,
	disableRightClickMenu: true,
	needSaveQuiz: {QUIZ_NEED_SAVE_QUIZ},
	saveQuizMode: {QUIZ_SAVE_QUIZ_MODE},
	saveQuizUrl: "{QUIZ_SAVE_QUIZ_URL}",
	allowRetakeQuiz: {QUIZ_ALLOW_RETAKE_QUIZ},
	maxNumRetake: {QUIZ_MAX_NUM_RETAKE},
	upperMarkForRetake: {QUIZ_UPPER_MARK_FOR_RETAKE},
	warnNeedRetake: {QUIZ_WARN_NEED_RETAKE},
	isQuizAnonymous: {QUIZ_IS_ANONYMOUS},
	needLogin: {QUIZ_NEED_LOGIN},
	reportNumOfColumns: {QUIZ_REPORT_NUM_COLUMNS},
	hideTitleBar: {QUIZ_HIDE_TITLEBAR},
	hideStatusBar: {QUIZ_HIDE_STATUSBAR},
	includeProfile: {QUIZ_INCLUDE_PROFILE},
	author: "{QUIZ_AUTHOR}",
	argument: "{QUIZ_ARGUMENT}",
	company: "{QUIZ_COMPANY}",
	quiz_date: "{QUIZ_DATE}",
	playSounds: {QUIZ_PLAY_SOUNDS},
	hasIntroText: {QUIZ_HAS_INTRO_TEXT},
	hasEpilogueText: {QUIZ_HAS_EPI_TEXT},
	askPrintAtTheEnd: {QUIZ_ASK_PRINT_AT_THE_END}
};

class Login
{
	constructor(name, email, hashPassword, sessionId)
	{
		this.name = name;
		this.email = email;
		this.password = hashPassword;
		this.sessionId = sessionId;
		this.otherFields = new Object();
	}
}

class Answer 
{
	constructor(choice,valuation,ansWeight,noAnsWeight, additionalText) 
	{
		this.choice = choice;
		this.valuation = valuation;
		this.ansWeight = ansWeight;
		this.noAnsWeight = noAnsWeight;
		this.additionalText = additionalText;
		this.shortTextAnswer = '';
		this.shortTextRemark = '';
		this.isGuess = false;
		this.score = 0;
	}
}

class Question 
{
	constructor(type, weight, numOfAnswers) 
	{
		this.typeOfQuestion = type;
		this.isSingleAns = false;
		this.weight = weight;
		this.answers = [];
		this.alreadyAnswered = false;
		this.nScore = 0;
		this.maxScore = 0;
		this.minScore = 0;
		this.nAnswers = numOfAnswers;
		this.valid = 0;
		this.shortTextQuestion = '';
		this.timeToAnswer = 0;
		this.noChoice = false;
	}
}

class Quiz
{
	constructor(numOfQuestions, startTime)
	{
		this.questions = new Array(numOfQuestions);
		this.currentQuestionPage = 1;
		this.isQuizCompleted = false;
		this.isQuizAbandoned = false;
		this.startTime = startTime;
		this.dateStartQuiz = new Date();
		this.dateCompleted = null;
		this.dateCompletedStr = '';
		this.time = 0;
		this.nRight = 0;
		this.nWrong = 0;
		this.nToDo = numOfQuestions;
		this.mark = 0;
		this.computeMarkErr = 0;
		this.userName = '';
		this.identityName = '';
		this.timeToAnswer = 0;

		this.ordineDomande = [];
		this.allSlidesIndex_before = new Array(numOfQuestions);
		this.allSlidesIndex_after = new Array(numOfQuestions);
		this.isQstDisplayed = new Array(numOfQuestions);
		for (var i=0; i<numOfQuestions; i++)
		{
			this.isQstDisplayed[i] = 0;
			this.allSlidesIndex_before[i] = new Array();
			this.allSlidesIndex_after[i] = new Array();
		}
		this.currentUser = null;
		this.numOfRetake = 0;
		this.options = options;
	}
}	
